ACC_bootsrap<- function(var_exp, var_obs, lon = NULL, lat = NULL, lonlatbox = NULL,nrand=1000,blocklen=c(1,1)) {
  # Matrix var_exp & var_obs should have dimensions (nexp/nobs, nsdates, 
  # nltimes, nlat, nlon).
  # ACC computes the Anomaly Correlation Coefficient for each jexp in 
  # 1:nexp and each jobs in 1:nobs which gives nexp x nobs ACC for each 
  # startdate and each leadtime.
  # A domain can be selected by providing the list of longitudes/latitudes
  # (lon/lat) of the grid together with the corner of the domain:
  #   lonlatbox = c(lonmin, lonmax, latmin, latmax)
  # 
  # Args:
  #   var_exp: Matrix of experimental data.
  #   var_obs: Matrix of observational data, same dimensions as var_exp except 
  #            along the first dimension.
  #   lon: Array of longitudes of the var_exp/var_obs grids, optional.
  #   lat: Array of latitudes of the var_exp/var_obs grids, optional.
  #   lonlatbox: Domain to select c(lonmin, lonmax, latmin, latmax), optional.
  
  # 
  # Returns:
  #   $ACC: Matrix with c(nexp, nobs, nsdates, nleadtimes, 4) dimensions.
  #        The fourth dimension of length 4 corresponds to the lower limit of
  #        the 95% confidence interval, the computed ACC, the upper limit of 
  #        the 95% confidence interval and the 95% significance level given by 
  #        a one-sided T-test.
  #   $MACC: Mean Anomaly Correlation Coefficient with c(nexp, nobs, nleadtimes) 
  #         dimensions.
  # 
  # History:
  #   1.0  #  2013-08  (C.Prodhomme, chloe.prodhomme@ic3.cat)  #  Original code
  
  dimsvar <- dim(var_exp)
  if (length(dimsvar) != 5) {
    stop("var_exp & var_obs should have dimensions (nexp/nsobs, nsdates, nltimes, nlat, nlon) ")
  }
  for (iind in 2:length(dimsvar)) {
    if (dim(var_obs)[iind] != dimsvar[iind]) {
      stop("var_exp & var_obs must have same dimensions except the first one (number of experiments or number of observational datasets) ")
    }
  }
  nexp <- dimsvar[1]
  nobs <- dim(var_obs)[1]
  nsdates <- dimsvar[2]
  nltimes <- dimsvar[3]
  nlat <- dimsvar[4]
  nlon <- dimsvar[5]
  
  if (length(blocklen) != 2)
    {
      stop("blocklen must be the size of size 2 (one size of blocks for experiment and obervations)")
    }

  #number of blocks needed to reconstruct the field
  nblon1=floor((nlon-1)/blocklen[1])+1
  nblon2=floor((nlon-1)/blocklen[2])+1
  nblat1=floor((nlat-1)/blocklen[1])+1
  nblat2=floor((nlat-1)/blocklen[2])+1
  distriACC <- array(NA, dim=c(nexp,nobs,nsdates,nltimes,nrand))
  distriMACC <- array(NA, dim=c(nexp,nobs,nltimes,nrand))

  for (irand in 1:nrand)
    {
      exprand <- array(NA, dim=c(nexp,nsdates,nltimes,nblat1,nblon1))
      obsrand <- array(NA, dim=c(nexp,nsdates,nltimes,nblat2,nblon2))
      
      vlonrand1 <- sample(1:(nlon-blocklen[1]), nblon1, replace=T)
      vlonrand2 <- sample(1:(nlon-blocklen[2]), nblon2, replace=T)
      
      vlatrand1 <- sample(1:(nlat-blocklen[1]), nblat1, replace=T)
      vlatrand2 <- sample(1:(nlat-blocklen[2]), nblat2, replace=T)

      for (iblock in 0:(blocklen[1]-1))
        {
          for (jblock in 0:(blocklen[1]-1))
            {
              exprand[, , ,seq(1,nlat,blocklen[1])+jblock ,seq(1,nlon,blocklen[1])+iblock]=
                var_exp[,,,vlatrand1+jblock,vlonrand1+iblock]
            }
        }
      for (iblock in 0:(blocklen[2]-1))
        {
          for (jblock in 0:(blocklen[2]-1))
            {
              obsrand[, , ,seq(1,nlat,blocklen[2])+jblock ,seq(1,nlon,blocklen[2])+iblock]=
                var_obs[, , ,vlatrand2+jblock,vlonrand2+iblock]
            }
        }

      exprand=InsertDim(exprand[, , , 1:nlat, 1:nlon],1,1)
      obsrand=InsertDim(obsrand[, , , 1:nlat, 1:nlon],1,1)
      ACC=ACC(exprand,obsrand)
      distriACC[, , , , irand]=(ACC$ACC)[, , , ,2]
      distriMACC[, , , irand]=(ACC$MACC)

    }
  ACC=apply(distriACC,c(1,2,3,4), function(x) quantile(x, c(0.05,0.95))[1])

}
        
